
#include "camel.h"


Distribution* createDistribution(){
	Distribution* myDist;
	myDist = (Distribution*)malloc(sizeof(Distribution));
	return myDist;
}


void SetCamel(int size, float m, float w,  Random* myRand,  double max, Distribution* myCamel){
	/*creates an ordered set of size random variables in a camel type distribution*/
	/*a camel distribution with two 'humps' is dependent on 
	the fraction of the probability mass distributed on the humps (m)
	and the fraction of the total width the sum of the hump widths occupies (w) */

    int  i;
	double uBound, lBound;

	myCamel->size = size;
	double uniformRandVal;

	uBound = max*(0.25 + (w/4));
	lBound = max*(0.25 - (w/4));


	for(i = 0; i < (int)(m* size)/2; i++){
		myCamel->dataList[i] = myRand->uniform(lBound, uBound);
	}

	uBound = max*(0.75 + (w/4));
	lBound = max*(0.75 - (w/4));

	for(i= (int)(m* size)/2; i < (int)(m* size); i++){
		myCamel->dataList[i] = myRand->uniform(lBound, uBound);
	}

	for (i = (int)(m* size); i < size; i++){
		uniformRandVal = myRand->uniform(0, max);
		myCamel->dataList[i] = uniformRandVal;
	}

}

void SetRectangle(int size, Random* myRand, double max, Distribution* myRect)
{
	int i;
	myRect->size = size;

	double uniformRandVal, minVal;
	minVal = 0;

	for (i = 0; i < size; i++){
		uniformRandVal = myRand->uniform(minVal, max);
		myRect->dataList[i] = uniformRandVal;
	}
}

void SetJitter(int size, Random* myRand, double max, Distribution* myJitter)
{
/*divides the interval (0,1) into ten intervals
	and determines uniformly distributed random numbers
		between those intervals*/

/*(0.0-0.1), (0.1-0.2), (0.2-0.3), (0.3-0.4), (0.4-0.5), 
  (0.5-0.6), (0.6-0.7), (0.7-0.8), (0.8-0.9), (0.9-1.0)*/

myJitter->size = size;
int intervalCount = size/10;
int i;
double minVal, maxVal, uniformRandVal;
minVal = 0;
maxVal = minVal + intervalCount;
int beginVal, endVal;
beginVal = 0;
endVal = intervalCount;

while(maxVal <=size){
	for(i = beginVal; i < endVal; i++){
		uniformRandVal = myRand->uniform(minVal, maxVal);
		myJitter->dataList[i] = uniformRandVal;
	}
beginVal = endVal;
endVal+= intervalCount;
minVal = maxVal;
maxVal+=intervalCount;
}

}

void SetTriangle(double LowerBound, double UpperBound, double PointX, int size, Random* myRand, Distribution* myTriangle, unsigned int Max)
{
/*triangle(A, B, C) is a triangular distribution
with a lower bound of A, an upper bound of B, and the x value of the
peak of the triangle = C */

/*this should work for positive and negative triangular distributions*/
/*randVarTriangular = A + sqrt(U*(B-A)*(C-A)) where 
U is a random number uniformly distributed between 0 and 1 if (C-A)/(B-A) >= U
	else randVarTriangular = B - sqrt((1-U)*(B-A)*(B-C))*/


myTriangle->size = size;
double  retval, mantissa1, mantissa2, uniformRandVal;

mantissa1 = (UpperBound - LowerBound) * (PointX - LowerBound);
mantissa2 = (UpperBound - LowerBound)*(UpperBound - PointX);
for(int i = 0; i < size; i++){
	uniformRandVal = myRand->uniform(LowerBound,UpperBound);
	if(((PointX - LowerBound)/(UpperBound-LowerBound)) >= uniformRandVal){
		retval = LowerBound + sqrt(uniformRandVal * mantissa1); 
	}else{
		retval = UpperBound - sqrt((1-uniformRandVal)* mantissa2);
	}

	myTriangle->dataList[i] = Max * retval;
}



}
